/*
* File         : SAP.cpp
* Author       : Pawel Roman
* Created      : Tue Aug 26 14:40:10 1997
* Revision     : $Revision$
* Updated      : $Date$
* Copyright: Northeast Parallel Architectures Center
*            at Syracuse University 1997
*/

#include "SAP.h"

#if !defined( _WIN32 )
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <netdb.h>
#else
#include <stdio.h>
#include <time.h>
#endif


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

unsigned long SAP::
avail() const
{
	unsigned long bytes;
#if defined( _WIN32 )
	if( ioctlsocket( handle_, FIONREAD, &bytes ) == SOCKET_ERROR )
#else
		if( ioctl( handle_, FIONREAD, &bytes ) )
#endif
			bytes = 0;
		
		return bytes;
}

int SAP::
status() const
{
	int st;
	char buffer;
	fd_set fset;
	timeval tv = { 0, 0 };
	FD_ZERO( &fset );
	FD_SET( handle_, &fset );
	st = select( handle_ + 1, &fset, NULL, NULL, &tv );
	if( st == 1 )
		return recv( handle_, &buffer, 1, MSG_PEEK );
	
	if( st == 0 ) 
		return 1;
	
	return -1;
}

int SAP::
open( int port, const char* host )
{
	struct sockaddr_in name;
	if( ( handle_ = socket( PF_INET, SOCK_STREAM, 0 ) ) < 0 )
	{
#if defined( _WIN32 )
		fprintf( stderr, "[!](SAP::open) cannot create a socket\n" );
#else
		perror( "[!](SAP::open)" );
#endif
		return -1;
	}
	
	name.sin_family = AF_INET;
	name.sin_port = htons( port );
	
	struct hostent* hostinfo;
	hostinfo = gethostbyname( host );
	if( ! hostinfo )
	{
#if defined( _WIN32 )	
		fprintf( stderr, "[!](SAP::open) cannot find specified host\n" );
#else
		fprintf( stderr, "(SAP::open) %s: %s\n", host, hstrerror( h_errno ) );
#endif
		return -1;
	}
	name.sin_addr = *( struct in_addr* )hostinfo->h_addr;
	if( connect( handle_, ( struct sockaddr* )&name, sizeof( name ) ) < 0 )
	{
#if defined( _WIN32 )
		fprintf( stderr, "[!](SAP::open) cannot connect to specified host\n" );
#else
		perror( "[!](SAP::open)" );
#endif
		return -1;
	}
	return 0;
}

int SAP::
open( int port )
{
	struct sockaddr_in name;
	if( ( handle_ = socket( PF_INET, SOCK_STREAM, 0 ) ) < 0 )
	{
#if defined( _WIN32 )
		fprintf( stderr, "[!](SAP::open) cannot create a socket\n" );
#else
		perror( "[!](SAP::open)" );
#endif
		return -1;
	}
	
	name.sin_family = AF_INET;
	name.sin_addr.s_addr = htonl( INADDR_ANY );
	name.sin_port = htons( port );
	if( bind( handle_, ( struct sockaddr* )&name, sizeof name ) < 0 )
	{
#if defined( _WIN32 )
		fprintf( stderr, "[!](SAP::open) cannot bind the socket\n" );
#else
		perror( "[!](SAP::open)" );
#endif
		return -1;
	}
	listen( handle_, 10 );
	return 0;
}

int SAP::
accept( SAP& new_sap )
{
	struct sockaddr_in name;
	int name_size = sizeof name;
	if( ( new_sap.handle_ = ::accept( handle_, ( struct sockaddr* )&name, &name_size ) ) < 0 )
		return -1;
	else
		return 0;
}

int SAP::
close() 
{ 
#if defined( _WIN32 )
	return closesocket( handle_ );
#else
	return ::close( handle_ );
#endif
}

int SAP::
recv_n( void* buffer, size_t size ) const
{
	int st;
	size_t len = size;
	do
	{
		st = ::recv( handle_, ( char* )buffer, len, 0 );
		if( st > 0 )
		{
			len -= st;
			buffer = ( char* )buffer + st;
		} 
		else
		{
			if( st < 0 )
				perror( "[!](SAP::recv_n)" );
			
			return st;
		}
	}
	while( len );
	return size;
}

int SAP::
recv_n( void* buffer, size_t size, int flags, timeval* tv ) const
{
	int st;
	size_t len = size;
	fd_set fset;
	FD_ZERO( &fset );
	FD_SET( handle_, &fset );
	time_t end_time = time( NULL ) + ( tv ? tv->tv_sec : 0 );
	do
	{
		if( ( st = select( handle_ + 1, &fset, NULL, NULL, tv ) ) < 0 ) 
		{
			perror( "[!](SAP::recv_n)" );
			return size - len;
		} 
		if( st == 0 )
			return size - len;
		
		st = ::recv( handle_, ( char* )buffer, len, flags );
		if( st > 0 )
		{
			len -= st;
			buffer = ( char* )buffer + st;
		} 
		else
		{
			if( st < 0 )
				perror( "[!](SAP::recv_n)" );
			
			return st;
		}
	}
	while( len &&( time( NULL ) < end_time ) );
	return size - len;
}

int SAP::
send_n( void* buffer, size_t size ) const
{
	int st;
	size_t len = size;
	do
	{
		st = ::send( handle_, ( char* )buffer, len, 0 );
		if( st > 0 )
		{
			len -= st;
			buffer = ( char* )buffer + st;
		} 
		else
		{
			if( st < 0 )
				perror( "[!](SAP::send_n)" );
			
			return st;
		}
	}
	while( len );
	return size;
}

int SAP::
init()
{
#if defined( _WIN32 )
	WORD wVersionRequested;
	WSADATA wsaData;
	wVersionRequested = MAKEWORD( 2, 0 ); 
	if( WSAStartup( wVersionRequested, &wsaData ) != 0 )
		return -1;
	
	if ( LOBYTE( wsaData.wVersion ) != 2 || HIBYTE( wsaData.wVersion ) != 0 )
	{
		WSACleanup();
		return -1;
	}
#endif
	return 0;
}
